@@ -1,7 +1,6 @@
 partial.rfsrc <- function(
   object,
   oob = TRUE,
-  m.target = NULL,
   partial.type = NULL,
   partial.xvar = NULL,
   partial.values = NULL,
@@ -17,6 +16,8 @@ partial.rfsrc <- function(
   user.option <- list(...)
   terminal.qualts <- is.hidden.terminal.qualts(user.option)
   terminal.quants <- is.hidden.terminal.quants(user.option)
+  data.pass <- is.hidden.data.pass(user.option)
+  experimental <- is.hidden.data.pass(user.option)
   ## Object consistency
   ## (TBD, TBD, TBD) Version checking is NOT implemented in this mode (TBD, TBD, TBD)
   if (missing(object)) {
@@ -58,6 +59,8 @@ partial.rfsrc <- function(
   if (length(which(xvar.names == partial.xvar)) != 1) {
     stop("x-variable specified incorrectly:  ", partial.xvar)
   }
+  ## uniquify the x-var
+  partial.values <- sort(unique(partial.values))
   ## verify the x-var2
   if (!is.null(partial.xvar2)) {   
       if (length(partial.xvar2) != length(partial.values2)) {
@@ -71,14 +74,11 @@ partial.rfsrc <- function(
   }
   ## Caution:  There are no checks on partial.type.  Note that "rel.freq" and "mort"
   ## are equivalent from a native code perspective.
-  ## Determine the immutable yvar factor map which is needed for
-  ## classification sexp dimensioning.  But, first convert object$yvar
-  ## to a data frame which is required for factor processing
-  #object$yvar <- as.data.frame(object$yvar)
-  #colnames(object$yvar) <- yvar.names
+  ## Determine the immutable yvar factor map
   yfactor <- object$yvar.factor
   ## multivariate family details
-  m.target.idx <- get.outcome.target(family, yvar.names, m.target)
+  ##m.target.idx <- get.outcome.target(family, yvar.names, NULL)
+  m.target.idx <- 1:length(yvar.names)
   ## short cut to get the y-outcome type and number of levels
   yvar.types <- yfactor$types
   yvar.nlevels  <- yfactor$nlevels
@@ -124,9 +124,11 @@ partial.rfsrc <- function(
   na.action.bits <- get.na.action(na.action)
   ## Initalize the high bits
   samptype.bits <- get.samptype(object$samptype)
-  partial.bits <- get.partial(length(partial.values))
+  partial.bits <- get.partial.bits(length(partial.values))
   terminal.qualts.bits <- get.terminal.qualts(terminal.qualts, object$terminal.qualts)
   terminal.quants.bits <- get.terminal.quants(terminal.quants, object$terminal.quants)
+  data.pass.bits <- get.data.pass(data.pass)
+  experimental.bits <- get.data.pass(experimental)
   seed <- get.seed(seed)
   do.trace <- get.trace(do.trace)
   ## Check that hdim is initialized.  If not, set it zero.
@@ -167,12 +169,15 @@ partial.rfsrc <- function(
                                       ensemble.bits +
                                       bootstrap.bits +
                                       cr.bits), 
+                                  ## note that no in-situ flag is passed here (TBD TBD)
                                   as.integer(
                                       samptype.bits +
                                       na.action.bits +
-                                        terminal.qualts.bits +
-                                          terminal.quants.bits +
-                                            partial.bits),
+                                      terminal.qualts.bits +
+                                      terminal.quants.bits +
+                                      partial.bits + 
+                                      data.pass.bits +
+                                      experimental.bits),
                                   ## >>>> start of maxi forest object >>>>
                                   as.integer(ntree),
                                   as.integer(n),
@@ -182,8 +187,7 @@ partial.rfsrc <- function(
                                        if (is.null(yvar.numeric.levels)) NULL else sapply(1:length(yvar.numeric.levels), function(nn) {as.integer(length(yvar.numeric.levels[[nn]]))}),
                                        if (is.null(subj)) NULL else as.integer(subj),
                                        if (is.null(event.type)) NULL else as.integer(length(event.type)),
-                                       if (is.null(event.type)) NULL else as.integer(event.type),
-                                       if (is.null(yvar.types)) NULL else as.double(as.vector(yvar))),
+                                       if (is.null(event.type)) NULL else as.integer(event.type)),
                                   if (is.null(yvar.numeric.levels)) {
                                       NULL
                                   }
@@ -191,11 +195,13 @@ partial.rfsrc <- function(
                                       lapply(1:length(yvar.numeric.levels),
                                              function(nn) {as.integer(yvar.numeric.levels[[nn]])})
                                   },
+                                  if (is.null(yvar.types)) NULL else as.double(as.vector(yvar)),
                                   list(as.integer(n.xvar),
                                        as.character(xvar.types),
                                        if (is.null(xvar.types)) NULL else as.integer(xvar.nlevels),
                                        if (is.null(xvar.numeric.levels)) NULL else sapply(1:length(xvar.numeric.levels), function(nn) {as.integer(length(xvar.numeric.levels[[nn]]))}),
-                                       as.double(as.vector(xvar))),
+                                       NULL,
+                                       NULL),
                                   if (is.null(xvar.numeric.levels)) {
                                       NULL
                                   }
@@ -203,6 +209,7 @@ partial.rfsrc <- function(
                                       lapply(1:length(xvar.numeric.levels),
                                              function(nn) {as.integer(xvar.numeric.levels[[nn]])})
                                   },
+                                  as.double(as.vector(xvar)),
                                   list(as.integer(length(case.wt)),
                                        if (is.null(case.wt)) NULL else as.double(case.wt),
                                        as.integer(sampsize),
@@ -210,7 +217,10 @@ partial.rfsrc <- function(
                                   list(if(is.null(event.info$time.interest)) as.integer(0) else as.integer(length(event.info$time.interest)),
                                        if(is.null(event.info$time.interest)) NULL else as.double(event.info$time.interest)),
                                   as.integer(object$totalNodeCount),
-                                  as.integer(object$seed),
+                                  as.integer(object$leafCount),
+                                  list(as.integer(object$seed),
+                                       if (is.null(object$seedVimp)) NULL else as.integer(object$seedVimp),
+                                       as.integer(object$optLoGrow)),
                                   as.integer(hdim),
                                   ## Object containing base learner settings, this is never NULL.
                                   object$base.learner, 
@@ -331,7 +341,7 @@ partial.rfsrc <- function(
                                     
                                   list(as.integer(0), NULL), ## Importance disabled.
                                   ## Partial variables enabled.  Note the as.integer is needed.
-                                  list(as.integer(get.type(family, partial.type)),
+                                  list(as.integer(get.partial.type(family, partial.type)),
                                        as.integer(which(xvar.names == partial.xvar)),
                                        as.integer(length(partial.values)),
                                        as.double(partial.values),
@@ -578,11 +588,144 @@ partial.rfsrc <- function(
         rfsrcOutput <- c(rfsrcOutput, regrOutput = list(regrOutput))
       }
     }
+  ## append other useful information here
+  rfsrcOutput$partial.values <- partial.values
+  rfsrcOutput$yvar.names <- object$yvar.names
+  rfsrcOutput$event.info <- object$event.info 
+  ## return the goodies
   class(rfsrcOutput) <- c("rfsrc", "partial",   family)
   return (rfsrcOutput)
 }
 partial <- partial.rfsrc
-get.partial <- function (partial.length) {
+##################################################################
+##
+##
+## work-horse for converting raw data to useable format for plots
+##
+##
+##################################################################
+get.partial.plot.data <- function(o, target, m.target = NULL) {
+  ##--------------------------------------------------------------
+  ##
+  ## coherency checks
+  ##
+  ##--------------------------------------------------------------
+  if (sum(inherits(o, c("rfsrc", "partial"), TRUE) == c(1, 2)) != 2) {
+    stop("this function only works for objects of class `(rfsrc, partial)'")
+  }
+  ##--------------------------------------------------------------
+  ##
+  ## extract the partial data, family specific
+  ##
+  ##--------------------------------------------------------------
+  family <- class(o)[3]
+  yvar.names <- o$yvar.names
+  ## multivariate families
+  if (family == "regr+" | family == "class+" | family == "mix+") {
+    ## determine the y-outcome to be used
+    if (missing(m.target)) {
+      m.target <- yvar.names[1]
+    }
+    else {
+      m.target <- match.arg(m.target, yvar.names)
+    }      
+    ## pull the data
+    pdta  <- unlist(list(o$classOutput, o$regrOutput), recursive = FALSE)[[m.target]]
+    ## convert to either regression or classification
+    ## classification
+    if (length(dim(pdta)) > 2) {
+      ## identify target class level
+      if (missing(target)) {
+        target <- 1
+      }
+      else {
+        target <- match(match.arg(target, yvar.names), yvar.names)
+      }
+      pdta <- pdta[, 1 + target, ]
+      pdta[is.infinite(pdta)] <- NA
+      yhat <- colMeans(pdta, na.rm = TRUE)    
+    }
+    ## regression
+    else {
+      pdta[is.infinite(pdta)] <- NA
+      yhat <- colMeans(pdta, na.rm = TRUE)
+    }
+  }
+  ## regression
+  else if (family == "regr") {
+    pdta <- o$regrOutput[[1]]
+    pdta[is.infinite(pdta)] <- NA
+    yhat <- colMeans(pdta, na.rm = TRUE)    
+  }
+  ## classification
+  else if (family == "class") {
+    pdta <- o$classOutput[[1]]
+    yvar.names <- colnames(pdta[,,1])[-1]
+    if (missing(target)) {
+      target <- 1
+    }
+    else {
+      target <- match(match.arg(target, yvar.names), yvar.names)
+    }
+    pdta <- pdta[, 1 + target, ]
+    pdta[is.infinite(pdta)] <- NA
+    yhat <- colMeans(pdta, na.rm = TRUE)    
+  }
+  ## survival
+  else if (family == "surv") {
+    pdta <- o$survOutput
+    pdta[is.infinite(pdta)] <- NA
+    ## surv, chf
+    if (length(dim(pdta)) > 2) {
+      yhat <- do.call(cbind, lapply(1:dim(pdta)[2], function(k) {
+        colMeans(pdta[, k, ], na.rm = TRUE)
+      }))
+    }
+    ## mortality
+    else {
+      yhat <- colMeans(pdta, na.rm = TRUE)
+    }
+  }
+  ## competing risk
+  else if (family == "surv-CR") {
+    pdta <- o$survOutput
+    pdta[is.infinite(pdta)] <- NA
+    ## identify target cause specific event
+    if (missing(target)) {
+        target <- 1
+      }
+      else {
+        target <- match(target, o$event.info$event.type)
+      }
+    ## cif, chf
+    if (length(dim(pdta)) > 3) {
+      yhat <- do.call(cbind, lapply(1:dim(pdta)[2], function(k) {
+        colMeans(pdta[, k, target, ], na.rm = TRUE)
+      }))
+    }
+    ## years-lost
+    else {
+      yhat <- colMeans(pdta[, target, ], na.rm = TRUE)
+    }
+  }
+  else {
+    stop("family specified is not supported")
+  }
+  ##--------------------------------------------------------------
+  ##
+  ## return the goodies
+  ##
+  ##--------------------------------------------------------------
+  list(x = o$partial.values, yhat = yhat, partial.time = o$partial.time)
+}
+##################################################################
+##
+##
+## utility functions
+##
+##
+##################################################################
+get.partial.bits <- function (partial.length) {
   ## Convert partial option into native code parameter.
   if (!is.null(partial.length)) {
     if (partial.length > 0) {
@@ -600,7 +743,7 @@ get.partial <- function (partial.length) {
     }
   return (bits)
 }
-get.type <- function (family, partial.type) {
+get.partial.type <- function (family, partial.type) {
   if (family == "surv") {
     ## The native code interprets "rel.freq" as "mort".  The R-side
     ## handles the difference downstream after native code exit.
